@@ -171,7 +171,7 @@ void list_all_decoders()
   std::cout << "JPEG decoders:\n";
   list_decoders(heif_compression_JPEG);
 
-  std::cout << "JPEG-2000 decoders:\n";
+  std::cout << "JPEG 2000 decoders:\n";
   list_decoders(heif_compression_JPEG2000);
 
 #if WITH_UNCOMPRESSED_CODEC
