@@ -26,8 +26,6 @@
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size)
 {
-  heif_init(nullptr);
-
   auto reader = std::make_shared<StreamReader_memory>(data, size, false);
 
   BitstreamRange range(reader, size);
@@ -44,7 +42,5 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size)
     box->dump(indent);
   }
 
-  heif_deinit();
-
   return 0;
 }
