@@ -1,6 +1,6 @@
 /*
  * HEIF codec.
- * Copyright (c) 2023, Dirk Farin <dirk.farin@gmail.com>
+ * Copyright (c) 2023 Dirk Farin <dirk.farin@gmail.com>
  *
  * This file is part of libheif.
  *
@@ -44,6 +44,11 @@ Op_RGB_to_YCbCr<Pixel>::state_after_conversion(const ColorState& input_state,
     return {};
   }
 
+  int matrix = target_state.nclx_profile.get_matrix_coefficients();
+  if (matrix == 8 || matrix == 11 || matrix == 14) {
+    return {};
+  }
+
   std::vector<ColorStateWithCost> states;
 
   ColorState output_state;
@@ -60,6 +65,7 @@ Op_RGB_to_YCbCr<Pixel>::state_after_conversion(const ColorState& input_state,
     output_state.chroma = target_state.chroma;
     output_state.has_alpha = input_state.has_alpha;  // we simply keep the old alpha plane
     output_state.bits_per_pixel = input_state.bits_per_pixel;
+    output_state.nclx_profile = target_state.nclx_profile;
 
     states.push_back({output_state, SpeedCosts_Unoptimized});
   }
@@ -70,6 +76,7 @@ Op_RGB_to_YCbCr<Pixel>::state_after_conversion(const ColorState& input_state,
     output_state.chroma = heif_chroma_444;
     output_state.has_alpha = input_state.has_alpha;  // we simply keep the old alpha plane
     output_state.bits_per_pixel = input_state.bits_per_pixel;
+    output_state.nclx_profile = target_state.nclx_profile;
 
     states.push_back({output_state, SpeedCosts_Unoptimized});
   }
@@ -81,6 +88,7 @@ Op_RGB_to_YCbCr<Pixel>::state_after_conversion(const ColorState& input_state,
 template<class Pixel>
 std::shared_ptr<HeifPixelImage>
 Op_RGB_to_YCbCr<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
+                                           const ColorState& input_state,
                                            const ColorState& target_state,
                                            const heif_color_conversion_options& options) const
 {
@@ -163,12 +171,10 @@ Op_RGB_to_YCbCr<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixel
   int matrix_coeffs = 2;
   RGB_to_YCbCr_coefficients coeffs = RGB_to_YCbCr_coefficients::defaults();
   bool full_range_flag = true;
-  if (target_state.nclx_profile) {
-    full_range_flag = target_state.nclx_profile->get_full_range_flag();
-    matrix_coeffs = target_state.nclx_profile->get_matrix_coefficients();
-    coeffs = get_RGB_to_YCbCr_coefficients(target_state.nclx_profile->get_matrix_coefficients(),
-                                           target_state.nclx_profile->get_colour_primaries());
-  }
+  full_range_flag = target_state.nclx_profile.get_full_range_flag();
+  matrix_coeffs = target_state.nclx_profile.get_matrix_coefficients();
+  coeffs = get_RGB_to_YCbCr_coefficients(target_state.nclx_profile.get_matrix_coefficients(),
+                                         target_state.nclx_profile.get_colour_primaries());
 
   int x, y;
 
@@ -209,9 +215,9 @@ Op_RGB_to_YCbCr<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixel
         }
         else {
           out_cb[(y / subV) * out_cb_stride + (x / subH)] = (Pixel) clip_f_u16(
-              ((in_b[y * in_b_stride + x] * 219.0f) / 256) + limited_range_offset, fullRange);
+              ((in_b[y * in_b_stride + x] * 224.0f) / 256) + limited_range_offset, fullRange);
           out_cr[(y / subV) * out_cb_stride + (x / subH)] = (Pixel) clip_f_u16(
-              ((in_r[y * in_b_stride + x] * 219.0f) / 256) + limited_range_offset, fullRange);
+              ((in_r[y * in_b_stride + x] * 224.0f) / 256) + limited_range_offset, fullRange);
         }
       }
       else {
@@ -294,14 +300,12 @@ Op_RRGGBBxx_HDR_to_YCbCr420::state_after_conversion(const ColorState& input_stat
     return {};
   }
 
-  if (input_state.nclx_profile) {
-    int matrix = input_state.nclx_profile->get_matrix_coefficients();
-    if (matrix == 0 || matrix == 8 || matrix == 11 || matrix == 14) {
-      return {};
-    }
-    if (!input_state.nclx_profile->get_full_range_flag()) {
-      return {};
-    }
+  int matrix = target_state.nclx_profile.get_matrix_coefficients();
+  if (matrix == 0 || matrix == 8 || matrix == 11 || matrix == 14) {
+    return {};
+  }
+  if (!target_state.nclx_profile.get_full_range_flag()) {
+    return {};
   }
 
   if (target_state.chroma != heif_chroma_420) {
@@ -319,6 +323,7 @@ Op_RRGGBBxx_HDR_to_YCbCr420::state_after_conversion(const ColorState& input_stat
   output_state.chroma = heif_chroma_420;
   output_state.has_alpha = input_state.has_alpha;  // we generate an alpha plane if the source contains data
   output_state.bits_per_pixel = input_state.bits_per_pixel;
+  output_state.nclx_profile = target_state.nclx_profile;
 
   states.push_back({output_state, SpeedCosts_Unoptimized});
 
@@ -328,6 +333,7 @@ Op_RRGGBBxx_HDR_to_YCbCr420::state_after_conversion(const ColorState& input_stat
 
 std::shared_ptr<HeifPixelImage>
 Op_RRGGBBxx_HDR_to_YCbCr420::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
+                                                const ColorState& input_state,
                                                 const ColorState& target_state,
                                                 const heif_color_conversion_options& options) const
 {
@@ -390,14 +396,10 @@ Op_RRGGBBxx_HDR_to_YCbCr420::convert_colorspace(const std::shared_ptr<const Heif
   int le = (input->get_chroma_format() == heif_chroma_interleaved_RRGGBBAA_LE ||
             input->get_chroma_format() == heif_chroma_interleaved_RRGGBB_LE) ? 1 : 0;
 
-  auto colorProfile = input->get_color_profile_nclx();
-  RGB_to_YCbCr_coefficients coeffs = RGB_to_YCbCr_coefficients::defaults();
-  bool full_range_flag = true;
-  if (colorProfile) {
-    full_range_flag = target_state.nclx_profile->get_full_range_flag();
-    coeffs = get_RGB_to_YCbCr_coefficients(colorProfile->get_matrix_coefficients(),
-                                           colorProfile->get_colour_primaries());
-  }
+  bool full_range_flag = target_state.nclx_profile.get_full_range_flag();
+  RGB_to_YCbCr_coefficients coeffs = get_RGB_to_YCbCr_coefficients(
+      target_state.nclx_profile.get_matrix_coefficients(),
+      target_state.nclx_profile.get_colour_primaries());
 
   for (int y = 0; y < height; y++) {
     for (int x = 0; x < width; x++) {
@@ -495,10 +497,9 @@ Op_RGB24_32_to_YCbCr::state_after_conversion(const ColorState& input_state,
     return {};
   }
 
-  if (target_state.nclx_profile) {
-    if (target_state.nclx_profile->get_matrix_coefficients() == 0) {
-      return {};
-    }
+  int matrix = target_state.nclx_profile.get_matrix_coefficients();
+  if (matrix == 0 || matrix == 8 || matrix == 11 || matrix == 14) {
+    return {};
   }
 
   std::vector<ColorStateWithCost> states;
@@ -509,6 +510,7 @@ Op_RGB24_32_to_YCbCr::state_after_conversion(const ColorState& input_state,
   output_state.chroma = target_state.chroma;
   output_state.has_alpha = target_state.has_alpha;
   output_state.bits_per_pixel = 8;
+  output_state.nclx_profile = target_state.nclx_profile;
 
   states.push_back({output_state, SpeedCosts_Unoptimized});
 
@@ -537,6 +539,7 @@ inline void set_chroma_pixels(uint8_t* out_cb, uint8_t* out_cr,
 
 std::shared_ptr<HeifPixelImage>
 Op_RGB24_32_to_YCbCr::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
+                                         const ColorState& input_state,
                                          const ColorState& target_state,
                                          const heif_color_conversion_options& options) const
 {
@@ -591,11 +594,9 @@ Op_RGB24_32_to_YCbCr::convert_colorspace(const std::shared_ptr<const HeifPixelIm
 
   RGB_to_YCbCr_coefficients coeffs = RGB_to_YCbCr_coefficients::defaults();
   bool full_range_flag = true;
-  if (target_state.nclx_profile) {
-    full_range_flag = target_state.nclx_profile->get_full_range_flag();
-    coeffs = get_RGB_to_YCbCr_coefficients(target_state.nclx_profile->get_matrix_coefficients(),
-                                           target_state.nclx_profile->get_colour_primaries());
-  }
+  full_range_flag = target_state.nclx_profile.get_full_range_flag();
+  coeffs = get_RGB_to_YCbCr_coefficients(target_state.nclx_profile.get_matrix_coefficients(),
+                                         target_state.nclx_profile.get_colour_primaries());
 
 
   int bytes_per_pixel = (has_alpha ? 4 : 3);
@@ -784,15 +785,11 @@ Op_RGB24_32_to_YCbCr444_GBR::state_after_conversion(const ColorState& input_stat
     return {};
   }
 
-  if (!target_state.nclx_profile) {
+  if (target_state.nclx_profile.get_matrix_coefficients() != 0) {
     return {};
   }
 
-  if (target_state.nclx_profile->get_matrix_coefficients() != 0) {
-    return {};
-  }
-
-  if (input_state.nclx_profile && !input_state.nclx_profile->get_full_range_flag()) {
+  if (!target_state.nclx_profile.get_full_range_flag()) {
     return {};
   }
 
@@ -804,6 +801,7 @@ Op_RGB24_32_to_YCbCr444_GBR::state_after_conversion(const ColorState& input_stat
   output_state.chroma = heif_chroma_444;
   output_state.has_alpha = target_state.has_alpha;
   output_state.bits_per_pixel = 8;
+  output_state.nclx_profile = target_state.nclx_profile;
 
   states.push_back({output_state, SpeedCosts_Unoptimized});
 
@@ -813,6 +811,7 @@ Op_RGB24_32_to_YCbCr444_GBR::state_after_conversion(const ColorState& input_stat
 
 std::shared_ptr<HeifPixelImage>
 Op_RGB24_32_to_YCbCr444_GBR::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
+                                                const ColorState& input_state,
                                                 const ColorState& target_state,
                                                 const heif_color_conversion_options& options) const
 {
@@ -855,8 +854,7 @@ Op_RGB24_32_to_YCbCr444_GBR::convert_colorspace(const std::shared_ptr<const Heif
   }
 
 
-  assert(target_state.nclx_profile);
-  assert(target_state.nclx_profile->get_matrix_coefficients() == 0);
+  assert(target_state.nclx_profile.get_matrix_coefficients() == 0);
   int bytes_per_pixel = (has_alpha ? 4 : 3);
 
   for (int y = 0; y < height; y++) {
