@@ -998,6 +998,9 @@ heif_property_id HeifFile::add_property(heif_item_id id, std::shared_ptr<Box> pr
 
 void HeifFile::add_orientation_properties(heif_item_id id, heif_orientation orientation)
 {
+  // Note: ISO/IEC 23000-22:2019(E) (MIAF) 7.3.6.7 requires the following order:
+  // clean aperture first, then rotation, then mirror
+
   int rotation_ccw = 0;
   heif_transform_mirror_direction mirror;
   bool has_mirror = false;
@@ -1013,7 +1016,7 @@ void HeifFile::add_orientation_properties(heif_item_id id, heif_orientation orie
       rotation_ccw = 180;
       break;
     case heif_orientation_flip_vertically:
-      mirror = heif_transform_mirror_direction_horizontal;
+      mirror = heif_transform_mirror_direction_vertical;
       has_mirror = true;
       break;
     case heif_orientation_rotate_90_cw_then_flip_horizontally:
@@ -1026,7 +1029,7 @@ void HeifFile::add_orientation_properties(heif_item_id id, heif_orientation orie
       break;
     case heif_orientation_rotate_90_cw_then_flip_vertically:
       rotation_ccw = 270;
-      mirror = heif_transform_mirror_direction_horizontal;
+      mirror = heif_transform_mirror_direction_vertical;
       has_mirror = true;
       break;
     case heif_orientation_rotate_270_cw:
