@@ -3,7 +3,7 @@
 
   MIT License
 
-  Copyright (c) 2019 struktur AG, Dirk Farin <farin@struktur.de>
+  Copyright (c) 2019 Dirk Farin <dirk.farin@gmail.com>
 
   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
@@ -30,7 +30,7 @@
 #include "libheif/pixelimage.h"
 
 // Enable for more verbose test output.
-#define DEBUG_ME 0
+constexpr bool kEnableDebugOutput = false;
 
 struct Plane {
   heif_channel channel;
@@ -223,6 +223,8 @@ std::vector<Plane> GetPlanes(const ColorState& state, int width, int height) {
 bool MakeTestImage(const ColorState& state, int width, int height,
                    HeifPixelImage* image) {
   image->create(width, height, state.colorspace, state.chroma);
+  auto target_nclx_profile = std::make_shared<color_profile_nclx>(state.nclx_profile);
+  image->set_color_profile_nclx(target_nclx_profile);
   std::vector<Plane> planes = GetPlanes(state, width, height);
   if (planes.empty()) return false;
   for (size_t i = 0; i < planes.size(); ++i) {
@@ -236,6 +238,22 @@ bool MakeTestImage(const ColorState& state, int width, int height,
   return true;
 }
 
+static bool NclxMatches(heif_colorspace colorspace,
+                        const color_profile_nclx& src_nclx,
+                        const color_profile_nclx& dst_nclx) {
+  if (colorspace != heif_colorspace_YCbCr) {
+    return true;
+  }
+
+  if (src_nclx.get_full_range_flag() != dst_nclx.get_full_range_flag() ||
+      src_nclx.get_matrix_coefficients() !=
+          dst_nclx.get_matrix_coefficients()) {
+    return false;
+  }
+
+  return true;
+}
+
 // Converts from 'input_state' to 'target_state' and checks that the resulting
 // image has the expected shape. If the reverse conversion is also supported,
 // does a round-trip back to the original state and checks the PSNR.
@@ -290,8 +308,7 @@ void TestConversion(const std::string& test_name, const ColorState& input_state,
 
   // Convert back in the other direction (if supported).
   ColorConversionPipeline reverse_pipeline;
-  if (reverse_pipeline.construct_pipeline(target_state, input_state,
-                                          options)) {
+  if (reverse_pipeline.construct_pipeline(target_state, input_state, options)) {
     INFO("reverse pipeline: " << reverse_pipeline.debug_dump_pipeline());
     std::shared_ptr<HeifPixelImage> recovered_image =
         reverse_pipeline.convert_image(out_image);
@@ -306,22 +323,28 @@ void TestConversion(const std::string& test_name, const ColorState& input_state,
          (input_state.chroma != heif_chroma_420 &&
           input_state.chroma != heif_chroma_422 &&
           target_state.chroma != heif_chroma_420 &&
-          target_state.chroma != heif_chroma_422));
-    double expected_psnr = expect_lossless ? 100. : 40.;
+          target_state.chroma != heif_chroma_422)) &&
+        NclxMatches(input_state.colorspace, input_state.nclx_profile,
+                    target_state.nclx_profile);
+    double expected_psnr = expect_lossless ? 100. : 38.;
 
     for (const Plane& plane : GetPlanes(input_state, width, height)) {
-      INFO("Channel: " << plane.channel << " (set DEBUG_ME to 1 in the code for more info)");
-#if DEBUG_ME
-      INFO("Original:\n" << PrintChannel(*in_image, plane.channel));
-      INFO("Recovered:\n" << PrintChannel(*recovered_image, plane.channel));
-      for (const Plane& converted_plane :
-           GetPlanes(target_state, width, height)) {
-        UNSCOPED_INFO("Converted channel "
-                      << converted_plane.channel << ":\n"
-                      << PrintChannel(*out_image, converted_plane.channel));
+      INFO("Channel: "
+           << plane.channel
+           << " (set kEnableDebugOutput to true in the code for more info)");
+      if (kEnableDebugOutput) {
+        UNSCOPED_INFO("Original:\n" << PrintChannel(*in_image, plane.channel));
+        UNSCOPED_INFO("Recovered:\n"
+                      << PrintChannel(*recovered_image, plane.channel));
+        for (const Plane& converted_plane :
+             GetPlanes(target_state, width, height)) {
+          UNSCOPED_INFO("Converted channel "
+                        << converted_plane.channel << ":\n"
+                        << PrintChannel(*out_image, converted_plane.channel));
+        }
       }
-#endif
-      double psnr = GetPsnr(*in_image, *recovered_image, plane.channel, expect_alpha_max);
+      double psnr =
+          GetPsnr(*in_image, *recovered_image, plane.channel, expect_alpha_max);
       CHECK(psnr >= expected_psnr);
     }
   }
@@ -331,9 +354,17 @@ void TestFailingConversion(const std::string& test_name,
                            const ColorState& input_state,
                            const ColorState& target_state,
                            const heif_color_conversion_options& options = {}) {
+  INFO(test_name);
+  INFO("downsampling=" << options.preferred_chroma_downsampling_algorithm
+                       << " upsampling="
+                       << options.preferred_chroma_upsampling_algorithm
+                       << " only_used_preferred="
+                       << (bool)options.only_use_preferred_chroma_algorithm);
   ColorConversionPipeline pipeline;
-  REQUIRE_FALSE(
-      pipeline.construct_pipeline(input_state, target_state, options));
+  bool construct_pipeline_res =
+      pipeline.construct_pipeline(input_state, target_state, options);
+  INFO("conversion pipeline: " << pipeline.debug_dump_pipeline());
+  REQUIRE_FALSE(construct_pipeline_res);
 }
 
 // Returns the list of valid has_alpha values for a given heif_chroma.
@@ -381,16 +412,45 @@ std::vector<int> GetValidBitsPerPixel(heif_chroma chroma) {
   }
 }
 
+// Returns a subset (to reduce the number of tests) of fully supported matrix
+// coefficients.
+const std::vector<heif_matrix_coefficients> GetSupportedMatrices() {
+  return {heif_matrix_coefficients_RGB_GBR,
+          heif_matrix_coefficients_SMPTE_240M};
+}
+
+// Returns matrix coefficients that are not currently supported by any operator.
+const std::vector<heif_matrix_coefficients> GetUnsupportedMatrices() {
+  return {heif_matrix_coefficients_SMPTE_ST_2085,
+          heif_matrix_coefficients_ICtCp};
+}
+
 // Returns of list of all valid ColorState (valid combinations
 // of a heif_colorspace/heif_chroma/has_alpha/bpp).
-std::vector<ColorState> GetAllColorStates() {
+std::vector<ColorState> GetAllColorStates(const std::vector<heif_matrix_coefficients>& matrices) {
   std::vector<ColorState> color_states;
   for (heif_colorspace colorspace : {heif_colorspace_YCbCr, heif_colorspace_RGB, heif_colorspace_monochrome}) {
     for (heif_chroma chroma : get_valid_chroma_values_for_colorspace(colorspace)) {
       for (bool has_alpha : GetValidHasAlpha(chroma)) {
         for (int bits_per_pixel : GetValidBitsPerPixel(chroma)) {
+          // Without nclx.
           ColorState color_state(colorspace, chroma, has_alpha, bits_per_pixel);
           color_states.push_back(color_state);
+
+          // With nclx.
+          if (colorspace == heif_colorspace_YCbCr) {
+            for (heif_matrix_coefficients matrix : matrices) {
+              for (bool full_range : {true, false}) {
+                color_state.nclx_profile.set_full_range_flag(full_range);
+                color_state.nclx_profile.set_matrix_coefficients(matrix);
+                color_state.nclx_profile.set_colour_primaries(
+                    heif_color_primaries_ITU_R_BT_709_5);
+                color_state.nclx_profile.set_transfer_characteristics(
+                    heif_color_primaries_SMPTE_240M);
+                color_states.push_back(color_state);
+              }
+            }
+          }
         }
       }
     }
@@ -417,11 +477,14 @@ TEST_CASE("All conversions", "[heif_image]") {
       .only_use_preferred_chroma_algorithm = only_use_preferred_chroma_algorithm};
 
   // Test all source and destination state combinations.
-  ColorState src_state = GENERATE(from_range(GetAllColorStates()));
-  ColorState dst_state = GENERATE(from_range(GetAllColorStates()));
-  // To debug a particular combination, hardcoe the ColorState values
+  ColorState src_state = GENERATE(
+      from_range(GetAllColorStates(GetSupportedMatrices())));
+  ColorState dst_state = GENERATE(
+      from_range(GetAllColorStates(GetSupportedMatrices())));
+  // To debug a particular combination, hardcode the ColorState values
   // instead:
   // ColorState src_state(heif_colorspace_YCbCr, heif_chroma_420, false, 8);
+  // src_state.nclx_profile.set_matrix_coefficients(...);
   // ColorState dst_state(...);
 
   bool require_supported = true;
@@ -441,6 +504,43 @@ TEST_CASE("All conversions", "[heif_image]") {
   TestConversion(os.str(), src_state, dst_state, options, require_supported);
 }
 
+TEST_CASE("Unsupported matrices", "[heif_image]") {
+  bool only_use_preferred_chroma_algorithm = GENERATE(false, true);
+  heif_chroma_downsampling_algorithm downsampling =
+      heif_chroma_downsampling_nearest_neighbor;
+  heif_chroma_upsampling_algorithm upsampling =
+      heif_chroma_upsampling_nearest_neighbor;
+  if (only_use_preferred_chroma_algorithm) {
+    downsampling = GENERATE(heif_chroma_downsampling_nearest_neighbor,
+                            heif_chroma_downsampling_average,
+                            heif_chroma_downsampling_sharp_yuv);
+    upsampling = GENERATE(heif_chroma_upsampling_nearest_neighbor,
+                          heif_chroma_upsampling_bilinear);
+  }
+  heif_color_conversion_options options = {
+      .preferred_chroma_downsampling_algorithm = downsampling,
+      .preferred_chroma_upsampling_algorithm = upsampling,
+      .only_use_preferred_chroma_algorithm = only_use_preferred_chroma_algorithm};
+
+  ColorState src_state =
+      GENERATE(from_range(GetAllColorStates(GetUnsupportedMatrices())));
+  ColorState dst_state =
+      GENERATE(from_range(GetAllColorStates(GetUnsupportedMatrices())));
+  color_profile_nclx default_nclx;
+
+  if (src_state == dst_state ||
+      NclxMatches(src_state.colorspace, src_state.nclx_profile,
+                  dst_state.nclx_profile) ||
+      (src_state.nclx_profile.get_matrix_coefficients() == default_nclx.get_matrix_coefficients() ||
+       dst_state.nclx_profile.get_matrix_coefficients() == default_nclx.get_matrix_coefficients())) {
+    return;
+  }
+
+  std::ostringstream os;
+  os << "from: " << src_state << "\nto:   " << dst_state;
+  TestFailingConversion(os.str(), src_state, dst_state, options);
+}
+
 TEST_CASE("Sharp yuv conversion", "[heif_image]") {
   heif_color_conversion_options sharp_yuv_options{
       .preferred_chroma_downsampling_algorithm =
